Package: CovEsts
Title: Nonparametric Estimators for Covariance Functions
Version: 1.1.0
Authors@R: 
    c(person(given = "Adam",
             family = "Bilchouris",
             role = c("cre", "aut"),
             email = "adam.bilchouris@gmail.com",
             comment = c(ORCID = "0009-0002-4649-0247")),
      person(given = "Andriy",
             family = "Olenko",
             role = "aut",
             email = "a.olenko@latrobe.edu.au",
             comment = c(ORCID = "0000-0002-0917-7000"))
     )
Description: Several nonparametric estimators of autocovariance functions. Procedures for constructing their confidence regions by using bootstrap techniques. Methods to correct autocovariance estimators and several tools for analysing and comparing them. Supplementary functions, including kernel computations and discrete cosine Fourier transforms. For more details see Bilchouris and Olenko (2025) <doi:10.17713/ajs.v54i1.1975>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: testthat (>= 3.0.0)
Imports: graphics, grDevices, parallel, stats, utils
Config/testthat/edition: 3
URL: https://github.com/AdamBilchouris/CovEsts
BugReports: https://github.com/AdamBilchouris/CovEsts/issues
NeedsCompilation: no
Packaged: 2026-04-19 09:55:40 UTC; Adam
Author: Adam Bilchouris [cre, aut] (ORCID:
    <https://orcid.org/0009-0002-4649-0247>),
  Andriy Olenko [aut] (ORCID: <https://orcid.org/0000-0002-0917-7000>)
Maintainer: Adam Bilchouris <adam.bilchouris@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-19 10:10:02 UTC
Built: R 4.5.2; ; 2026-04-20 23:21:08 UTC; unix
