# Generated by roxygen2: do not edit by hand

S3method(area_between,CovEsts)
S3method(area_between,default)
S3method(as.double,CovEsts)
S3method(check_pd,CovEsts)
S3method(check_pd,default)
S3method(hilbert_schmidt,CovEsts)
S3method(hilbert_schmidt,default)
S3method(kernel_est,CovEsts)
S3method(kernel_est,default)
S3method(lines,BootEsts)
S3method(lines,CovEsts)
S3method(lines,VarioEsts)
S3method(make_pd,CovEsts)
S3method(make_pd,default)
S3method(max_distance,CovEsts)
S3method(max_distance,default)
S3method(mse,CovEsts)
S3method(mse,default)
S3method(nearest_pd,CovEsts)
S3method(nearest_pd,default)
S3method(normalise_acf,CovEsts)
S3method(normalise_acf,default)
S3method(plot,BootEsts)
S3method(plot,CovEsts)
S3method(plot,VarioEsts)
S3method(print,BootEsts)
S3method(print,CovEsts)
S3method(print,VarioEsts)
S3method(shrinking,CovEsts)
S3method(shrinking,default)
S3method(spectral_norm,CovEsts)
S3method(spectral_norm,default)
S3method(splines_est,CovEsts)
S3method(splines_est,default)
S3method(to_pacf,CovEsts)
S3method(to_pacf,default)
S3method(to_vario,CovEsts)
S3method(to_vario,default)
export(adjusted_est)
export(area_between)
export(block_bootstrap)
export(bootstrap_sample)
export(check_pd)
export(corrected_est)
export(dct_1d)
export(hilbert_schmidt)
export(idct_1d)
export(kernel)
export(kernel_ec)
export(kernel_est)
export(kernel_symm)
export(kernel_symm_ec)
export(make_pd)
export(max_distance)
export(mse)
export(nearest_pd)
export(normalise_acf)
export(shrinking)
export(spectral_norm)
export(splines_est)
export(standard_est)
export(starting_locs)
export(taper)
export(tapered_est)
export(to_pacf)
export(to_vario)
export(truncated_est)
export(window)
export(window_ec)
export(window_symm)
export(window_symm_ec)
importFrom(grDevices,rgb)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,polygon)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,makePSOCKcluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(stats,acf)
importFrom(stats,optim)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,toeplitz)
importFrom(utils,head)
importFrom(utils,tail)
